package main

import (
	"github.com/spf13/cobra"

	"github.com/ovh/cds/cli"
	"github.com/ovh/cds/sdk"
)

var projectVariableCmd = cli.Command{
	Name:  "variable",
	Short: "Manage CDS project variables",
}

func projectVariable() *cobra.Command {
	return cli.NewCommand(projectVariableCmd, nil, []*cobra.Command{
		cli.NewCommand(projectVariableCreateCmd, projectCreateVariableRun, nil, withAllCommandModifiers()...),
		cli.NewListCommand(projectVariableListCmd, projectListVariableRun, nil, withAllCommandModifiers()...),
		cli.NewGetCommand(projectVariableShowCmd, projectVariableShowRun, nil, withAllCommandModifiers()...),
		cli.NewCommand(projectVariableDeleteCmd, projectDeleteVariableRun, nil, withAllCommandModifiers()...),
		cli.NewCommand(projectVariableUpdateCmd, projectUpdateVariableRun, nil, withAllCommandModifiers()...),
	})
}

var projectVariableCreateCmd = cli.Command{
	Name:  "add",
	Short: "Add a new variable on project. Variable type can be one of password, text, string, key, boolean, number, repository",
	Ctx: []cli.Arg{
		{Name: _ProjectKey},
	},
	Args: []cli.Arg{
		{Name: "variable-name"},
		{Name: "variable-type"},
		{Name: "variable-value"},
	},
}

func projectCreateVariableRun(v cli.Values) error {
	variable := &sdk.Variable{
		Name:  v["variable-name"],
		Type:  v["variable-type"],
		Value: v["variable-value"],
	}
	return client.ProjectVariableCreate(v[_ProjectKey], variable)
}

var projectVariableListCmd = cli.Command{
	Name:  "list",
	Short: "List CDS project variables",
	Ctx: []cli.Arg{
		{Name: _ProjectKey},
	},
}

func projectListVariableRun(v cli.Values) (cli.ListResult, error) {
	variables, err := client.ProjectVariablesList(v[_ProjectKey])
	if err != nil {
		return nil, err
	}
	return cli.AsListResult(variables), nil
}

var projectVariableDeleteCmd = cli.Command{
	Name:  "delete",
	Short: "Delete CDS project variable",
	Ctx: []cli.Arg{
		{Name: _ProjectKey},
	},
	Args: []cli.Arg{
		{Name: "variable-name"},
	},
}

func projectDeleteVariableRun(v cli.Values) error {
	return client.ProjectVariableDelete(v[_ProjectKey], v["variable-name"])
}

var projectVariableShowCmd = cli.Command{
	Name:  "show",
	Short: "Show a CDS project variable",
	Ctx: []cli.Arg{
		{Name: _ProjectKey},
	},
	Args: []cli.Arg{
		{Name: "variable-name"},
	},
}

func projectVariableShowRun(v cli.Values) (interface{}, error) {
	return client.ProjectVariableGet(v[_ProjectKey], v["variable-name"])
}

var projectVariableUpdateCmd = cli.Command{
	Name:  "update",
	Short: "Update CDS project variable value",
	Ctx: []cli.Arg{
		{Name: _ProjectKey},
	},
	Args: []cli.Arg{
		{Name: "variable-oldname"},
		{Name: "variable-name"},
		{Name: "variable-type"},
		{Name: "variable-value"},
	},
}

func projectUpdateVariableRun(v cli.Values) error {
	variable, err := client.ProjectVariableGet(v[_ProjectKey], v["variable-oldname"])
	if err != nil {
		return err
	}
	variable.Name = v["variable-name"]
	variable.Type = v["variable-type"]
	variable.Value = v["variable-value"]
	return client.ProjectVariableUpdate(v[_ProjectKey], variable)
}
